;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : DrGeo-menus.scm
;; DESCRIPTION : Menus for the drgeo plugin including some Scheme constructs
;; COPYRIGHT   : (C) 2003 Hilaire Fernandes
;;               (C) Adrian Ulises Soto 2004,2005, 2012
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. 
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;The following works, but it makes texmacs take years to load and
; it seems not to know where the directory is.  Maybe could be
; be fixed using a delayed version, because init-shell does not
; know where the directory is...

;;(define show-version
;;   (lambda ()
;;     (insert (plugin-eval "shell" "default" "drgeo --version"))))

(define show-example
  (lambda ()
    (load-in-new-window "../plugins/DrGeo/doc/pluginexample.tm")))

(define show-instructions
   (lambda ()
     (load-in-new-window  "../plugins/DrGeo/doc/plugindoc.tm")))

(menu-extend texmacs-extra-menu
  (if (equal? (get-env "prog-language") "DrGeo")
      (=> "DrGeo" (link DrGeo-menu))))

(menu-bind DrGeo-menu
  ("Instructions in new window" (show-instructions))
  ("Example in new window" (show-example))
  ---
  (-> "Common Scheme constructs"
     (-> "Assignements"
         (-> "define" 
             ("Define variable"
                (insert "(define <var> <exp>)"))
	     ("Define procedure"
	        (insert "(define (<procedure_name> <var_1> <etc>) <exp_1> <exp_2> <etc>)"))
	     ("Define procedure with variable number of arguments"
	        (insert "(define (<procedure_name> . <var_r>) <exp_1> <etc>)"))
             ("Define procedure with variable number of arguments"
                (insert "(define (<procedure_name> <var_1> <etc> . <var_r>) <exp_1> <exp_2> <etc>)")))
         ("let"
            (insert "(let ((<var_1> <exp_1>) <etc>) <exp1> <etc>)"))
         ("let*"
            (insert "(let* ((<var_1> <exp_1>) <etc>) <exp1> <etc>)"))
         ("letrec"
            (insert "(letrec((<function> (lambda (<var> <etc>) <exp1> <etc>))<etc>) <exp1> <etc>)"))
         ("set!"
            (insert "(set! <var> <exp>)")))
     (-> "Loops"
         ("Do"
            (insert "(do ((<loop_variable> <initial_value> <update_expresion>)<etc>)(<test_to_finish> <exp_1_after_finish> <etc>) <exp_1> <etc>)"))
         ("named let (for iteration)"
            (insert "(let <loop_name> ((<loop_variable> <initial_value>)<etc>) (if <test> (begin <exp_1> <etc> (loop_name <new_value>))))"))
         ("named let (general)"
            (insert "(let <loop_name> ((<loop_variable> <initial_value>)<etc>)<exp1> <etc>)"))
         ("while"
            (insert "(while <test> <exp_1> <etc>)"))
         ("for-each"
            (insert "(for-each <function> <list_1> <etc>)"))
         ("map (returns a list)"
            (insert "(map <function <list_1> <etc>)"))
         ("apply"
            (insert "(apply <procedure> <obj_1> <etc> <list_1> <etc>")))
     (-> "Program flow"
         ("If"
           (insert "(if <test> <exp_if_true> <exp_if_false>"))
         ("If (a common variant)"
           (insert "(if <test> (begin <exp1> <etc>) (begin <exp1> <etc>))"))
         ("cond"
           (insert "(cond ((<test_1> <exp_1> <etc>)<etc>)"))
         ("case"
           (insert "(case <expresion> (<key> <exp_1>)<etc>)"))
         (-> "other"
             ("cond"
               (insert "(cond <clause_1> <etc>)"))
             (-> "clauses for cond"
                 ("(test)"
                   (insert "(<test>)"))
                 ("(<test> <exp>)"
                   (insert "(<test> <exp_1> <etc>)"))
                 ("(<test> => <procedure>)"
                   (insert "(<test> => <procedure>)"))
                 ("(else <exp_1> <etc>)"
                   (insert "(else <exp_1> <etc>)")))))
     (-> "functions"
         ("lambda"
           (insert "(lambda (<var_1> <etc>)<exp_1> <etc>)")))
     (-> "tests"
         ("eq?"
           (insert "(eq? <obj_1> <obj_2>)"))
         ("eqv?"
           (insert "(eqv? <obj_1> <obj_2>)"))
         ("equal?"
           (insert "(equal? <obj_1> <obj_2>)"))
         ("null?"
           (insert "(null? <obj>)"))
         ("positive?"
           (insert "(positive? <obj>)"))
         ("list?"
           (insert "(list? <obj>)"))
         ("symbol?"
           (insert "(symbol? <obj>)"))
         ("boolean?"
           (insert "(boolean? <obj>)"))
         ("pair?"
           (insert "(pair? <obj>)"))
         ("number?"
           (insert "(number? <obj>)"))
         ("integer?"
           (insert "(integer? <obj>)"))
         ("real?"
           (insert "(real? <obj>)"))
         ("complex?"
           (insert "(complex? <obj>)"))
         ("char?"
           (insert "(char? <obj>)"))
         ("string?"
           (insert "(string? <obj>)"))
         ("vector?"
           (insert "(vector? <obj>)"))
         ("symbol?"
           (insert "(symbol? <obj>)"))
         ("procedure?"
           (insert "(procedure? <obj>)"))
         ("zero?"
           (insert "(zero? <obj>)")))
     (-> "transformations"
         ("string->list"
           (insert "(string->list <string>)"))
         ("string->number"
           (insert "(string->number <string>"))
         ("string->number (base n)"
           (insert "(string->number <string> <base>)"))
         ("string->symbol"
           (insert "(string->symbol <string>)"))
         ("symbol->string"
           (insert "(symbol->string <symbol>)"))
         ("vector->list"
           (insert "(vector->list <vector>)"))
         ("number->string"
           (insert "(number? <obj>)"))
         ("number->string (base n)"
           (insert "(number->string <number> <base>)"))
         ("list->string"
           (insert "(list->string <list>)"))
         ("list->vector"
           (insert "(list->vector <list>)"))))

  ---
  ("Newfigure" (insert " (new-figure \"<figuretitle>\")"))
  ---
  ("Lets (caption=name)" (insert "(lets <object> \"<name>\"  <etc>)"))
  ("Define(caption could be different)" (insert "(define <name> (<object> \"<caption>\" <etc>))"))
  ---
  (-> "Points" 
     ("Given by their coordinates" 
        (insert " (Point \"<caption>\" 'free <captionx> <captiony>)"))
     ("Point on a curve"
        (insert " (Point \"<caption>\" 'on-curve <curve> <number>)"))
     ("Midpoint of two points" 
        (insert " (Point \"<caption>\" 'middle-2pts <Point1> <Point2>)"))
     ("Midpoint of a segment" 
        (insert " (Point \"<caption>\" 'middle-segment <Segment>)"))
     ("Intersection of two curves"
        (insert " (Point \"<caption>\" 'intersection <curve1> <curve2>)"))
     ("Second intersection of two curves"
        (insert " (Point \"<caption>\" 'intersection2 <curve1> <curve2>)")))
  (-> "Straight Lines" 
     ("Passing through two points" 
        (insert " (Line \"<caption>\" '2points <point1> <point2>)"))
     ("Passing trough a point and parallel to a segment, vector, ray or line" 
        (insert " (Line \"<caption>\" 'parallel <point> <direction>)"))
     ("Passing trough a point and perpendicular to a segment, vector, ray or line" 
        (insert " (Line \"<caption>\" 'perpendicular <point> <direction>)")))
  (-> "Angles"
     ("Given three points"
       (insert " (Angle \"<caption>\" 'geometric <point1> <point2> <point3>)")))
  (-> "Rays" 
     ("From one point towards another" 
        (insert " (Ray \"<caption>\" '2points <point1> <point2>)")))
  (-> "Segments"
     ("Defined by its extremities"
        (insert " (Segment \"<caption>\" 'extremities <point1> <point2>)")))
  (-> "Vectors"
     ("Defined by two points"
        (insert " (Vector \"<caption>\" '2points <point1> <point2>)")))
  (-> "Numeric Objects"
     ("Given its value"
        (insert " (Numeric \"<caption>\" 'free <captionx> <captiony> <number>)"))
     ("Given a SEGMENT returns LENGTH"
        (insert " (Numeric \"<caption>\" 'segment-length <captionx> <captiony> <segment>)"))
     ("Given a VECTOR returns the NORM of the vector"
        (insert " (Numeric \"<caption>\" 'vector-norm <captionx> <captiony> <vector>)"))
     ("Given a POINT and a LINE returns DISTANCE"
        (insert " (Numeric \"<caption>\" 'point-line <captionx> <captiony> <point> <line>)"))
     ("Given a POINT and a CIRCLE returns DISTANCE"
        (insert " (Numeric \"<caption>\" 'point-circle <captionx> <captiony> <point> <circle>)"))
     ("Given a POINT and a POINT returns DISTANCE"
        (insert " (Numeric \"<caption>\" 'point-point <captionx> <captiony> <Point1> <Point2>)"))
     ("Given a CIRCLE returns LENGTH"
        (insert " (Numeric \"<caption>\" 'circle-length <captionx> <captiony> <circle>)"))
     ("Given a LINE returns SLOPE"
        (insert " (Numeric \"<caption>\" 'line-slope <captionx> <captiony> <line>)"))
     ("Given an ARC returns LENGTH"
        (insert " (Numeric \"<caption>\" 'arc-length <captionx> <captiony> <arc>)")))
  (-> "Circles"
     ("With a given center and passing through a point"
        (insert " (Circle \"<caption>\" '2points <point1> <point2>)"))
     ("With a given center and a radius given by a Numeric object"
        (insert " (Circle \"<caption>\" 'center-radius <point> <numeric>)"))
     ("With a given center and a radius given by a Segment"
        (insert " (Circle \"<caption>\" 'center-segment <point> <segment>)")))
  (-> "Arcs of a circle"
     ("Passing throuh three given points"
        (insert " (Arc \"<caption>\" '3points <point1> <point2> <point3>)")))
  (-> "Locus of points"
     ("Given by an independent point and a dependent point"
        (insert " (Locus \"<caption>\" '2points <independentPoint> <dependentPoint>)")))
  (-> "Polygon"
     ("Passing through n-points"
        (insert " (Polygon \"<caption>\" 'npoints <point1> <point2> <etc>)")))
  (-> "Geometric Transformations"
     ("Translation by a vector"
        (insert " (<newobject> \"<caption>\" 'translation <oldobject> <vector>)"))
     ("Rotation given a center and an angle"
        (insert " (<newobject> \"<caption>\" 'rotation <oldobject> <Point> <angle>)"))
     ("Dilatation(Scale, Homothecy) given a center and a Numeric object (factor)"
        (insert " (<newobject> \"<caption>\" 'scale <oldobject> <point> <numericobject>)"))
     ("Symmetry (Half-turn) given a center"
        (insert " (<newobject> \"<caption>\" 'symmetry <oldobject> <point>)"))
     ("reflection on a line"
        (insert " (<newobject> \"<caption>\" 'reflection <oldobject> <line>)")))
  (-> "Change styles of objects"
     ("Changing color"
        (insert " (send <object> 'color <color>)"))
     ("Changing thickness of curves"
        (insert " (send <object> 'thickness <thickness>)"))
     ("Changing point shape"
        (insert " (send <object> 'shape <shape>)"))
     ("Changing point size"
        (insert " (send <object> 'size <size>)"))
     (-> "Changing visibility"
        ("Make it invisible" 
           (insert " (send <object> 'masked)"))
        ("Make it visible"
           (insert " (send <object> 'unmasked)")))
     (-> "Available colors"
         ("black=0" (insert "black"))
         ("dark-grey=1" (insert "dark-grey"))
         ("grey=2" (insert "grey"))
         ("white=3" (insert "white"))
         ("dark-green=4" (insert "dark-green"))
         ("green=5" (insert "green"))
         ("dark-blue=6" (insert "dark-blue"))
         ("blue=7" (insert "blue"))
         ("bordeaux=8" (insert "bordeaux"))
         ("red=9" (insert "red"))
         ("orange=10" (insert "orange"))
         ("yellow=11" (insert "yellow")))
     (-> "Available sizes and thickness"
         ("dashed=0" (insert "dashed"))
         ("small=0"  (insert "small"))
         ("normal=1" (insert "normal"))
	 ("large=2"  (insert "large")))
     (-> "Available shapes"
         ("round=0" (insert "round"))
         ("cross=1" (insert "cross"))
         ("rectangle=2" (insert "rec"))
         ("round-empty=3" (insert "round-empty"))
         ("rectangle-empty=4" (insert "rec-empty")))))


